package com.gxx.sharding.service.impl;

import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.gxx.sharding.base.dao.UserMapper;
import com.gxx.sharding.base.vo.User;
import com.gxx.sharding.service.UserService;

/**
 * <dl>
 *    <dt><b>Title:</b></dt>
 *    <dd>
 *    	用户服务实现类
 *    </dd>
 *    <dt><b>Description:</b></dt>
 *    <dd>
 *    	<p>none
 *    </dd>
 * </dl>
 *
 * @author Administrator
 * @version 1.0, 2015年6月18日
 * @since sharding
 *
 */
@Service("userService")
public class UserServiceImpl implements UserService {

	@Autowired
	private UserMapper userDao;

	/**
	 * 新增用户
	 * @param user
	 */
	public void doSaveUser(User user) {
		userDao.insert(user);
	}
	
	/**
	 * 修改用户
	 * @param user
	 */
	public int doUpdateUser(User user) {
		return userDao.updateByPrimaryKey(user);
	}
	
	/**
	 * 删除用户
	 * @param user
	 */
	public int doDeleteUser(User user) {
		return userDao.deleteByPrimaryKey(user.getUserId());
	}
	
	/**
	 * 根据id查询用户
	 * @param id
	 * @return
	 */
	public User getUserByUserId(long userId) {
		return userDao.selectByPrimaryKey(userId);
	}
	
	/**
	 * 根据id集合查询用户
	 * @param userIdList
	 * @return
	 */
	public List<User> queryUsersInUserId(List<Long> userIdList) {
		return userDao.queryUsersInUserId(userIdList);
	}
	
	/**
	 * 根据id区间查询用户
	 * @param beginUserId
	 * @param endUserId
	 * @return
	 */
	public List<User> queryUsersBetweenUserId(Long beginUserId, Long endUserId) {
		return userDao.queryUsersBetweenUserId(beginUserId, endUserId);
	}

	/**
	 * 计数
	 * @return
	 */
	public long countUser() {
		return userDao.countUser();
	}

	/**
	 * 求和
	 * @return
	 */
	public long sumUser() {
		return userDao.sumUser();
	}

	/**
	 * 取小
	 * @return
	 */
	public long minUser() {
		return userDao.minUser();
	}

	/**
	 * 取大
	 * @return
	 */
	public long maxUser() {
		return userDao.maxUser();
	}

	/**
	 * 平均
	 * @return
	 */
	public double avgUser() {
		return userDao.avgUser();
	}
	
	/**
	 * 根据limit区间查询用户
	 * @param beginUserId
	 * @param num
	 * @return
	 */
	public List<User> queryUsersLimitUserId(Long beginUserId, Long num) {
		return userDao.queryUsersLimitUserId(beginUserId, num);
	}

//	/**
//	 * 根据姓名查用户
//	 * @param name
//	 * @return
//	 */
//	public User getUserByName(String name) {
//		return userDao.getUserByName(name);
//	}
}
