package com.gxx.sharding.web.user;

import java.util.ArrayList;
import java.util.List;

import javax.servlet.http.HttpServletRequest;

import org.apache.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.ResponseBody;

import com.gxx.sharding.base.utils.DateUtils;
import com.gxx.sharding.base.vo.User;
import com.gxx.sharding.dto.UserDto;
import com.gxx.sharding.service.UserService;

/**
 * QueryAllocateController负责查询调拨
 * 
 * @author gxx
 */
@Controller
@RequestMapping("/user/")
public class UserController {
	/**
	 * 日志处理器
	 */
	private final Logger logger = Logger.getLogger(UserController.class);
	
	@Autowired
	private UserService userService;
	
	@RequestMapping(value = "/preRegistFtl", method = RequestMethod.GET)
	public String preRegistFtl() {
		return "user/preRegistFtl";
	}
	
	@RequestMapping(value = "/preRegistJsp", method = RequestMethod.GET)
	public String preRegistJsp() {
		return "user/preRegistJsp";
	}
	
	@RequestMapping(value = "/preQueryFtl", method = RequestMethod.GET)
	public String preQueryFtl() {
		return "user/preQueryFtl";
	}
	
	/**
	 * 注册
	 * @param request
	 * @param userDto
	 * @return
	 */
	@RequestMapping(value = "/registJsp",produces="application/json")
	public @ResponseBody UserDto registJsp(HttpServletRequest request, UserDto userDto) {
		logger.info("用户注册：姓名[" + userDto.getName() + "]，密码[" + userDto.getPassword() + "]");
		/**
		 * 1.判用户名是否存在
		 */
		User user = null;//userService.getUserByName(userDto.getName());
		if(user != null){
			userDto.setSuccess(Boolean.FALSE.booleanValue());
			userDto.setMessage("用户名[" + userDto.getName() + "]已存在!");
			return userDto;
		}
		/**
		 * 2.创建用户对象 并 新增用户
		 */
		user = new User();
		user.setUserId(userDto.getId().longValue());
		user.setName(userDto.getName());
		user.setPassword(userDto.getPassword());
		user.setCreateDate("20150618");
		user.setCreateTime("000000");
		userService.doSaveUser(user);
		/**
		 * 3.返回结果
		 */
		userDto.setSuccess(Boolean.TRUE.booleanValue());
		userDto.setMessage("注册成功!");
		return userDto;
	}
	
	/**
	 * 注册
	 * @param request
	 * @param userDto
	 * @return
	 */
	@RequestMapping(value = "/registFtl")
	public String registFtl(HttpServletRequest request, UserDto userDto) {
		logger.info("用户注册：姓名[" + userDto.getName() + "]，密码[" + userDto.getPassword() + "]");
		/**
		 * 1.判用户名是否存在
		 */
		User user = null;//userService.getUserByName(userDto.getName());
		if(user != null){
			userDto.setSuccess(Boolean.FALSE.booleanValue());
			userDto.setMessage("用户名[" + userDto.getName() + "]已存在!");
			return "user/result";
		}
		/**
		 * 2.创建用户对象 并 新增用户
		 */
		user = new User();
		user.setUserId(userDto.getId().longValue());
		user.setName(userDto.getName());
		user.setPassword(userDto.getPassword());
		user.setCreateDate("20150618");
		user.setCreateTime("000000");
		userService.doSaveUser(user);
		/**
		 * 3.返回结果
		 */
		userDto.setSuccess(Boolean.TRUE.booleanValue());
		userDto.setMessage("注册成功!");
		return "user/result";
	}
	
	/**
	 * 查询
	 * @param request
	 * @param userDto
	 * @return
	 */
	@RequestMapping(value = "/queryFtl")
	public String queryFtl(HttpServletRequest request, UserDto userDto) {
		logger.info("用户查询：id值[" + userDto.getIdValue() + "]，查询类型[" + userDto.getType() + "]");
		
		String message = "查询成功！";
		
		/**
		 * 1.根据查询类型区分
		 */
		if("equal".equals(userDto.getType())) {
			User user = userService.getUserByUserId(Long.parseLong(userDto.getIdValue()));
			logger.info("根据id[" + userDto.getIdValue() + "]查询用户:" + user);
			message += user;
		} else if("update".equals(userDto.getType())) {
			User user = userService.getUserByUserId(Long.parseLong(userDto.getIdValue()));
			user.setCreateDate(DateUtils.getCurrentDate());
			user.setCreateTime(DateUtils.getCurrentTime());
			userService.doUpdateUser(user);
			logger.info("根据id[" + userDto.getIdValue() + "]修改用户:" + user);
			message += user;
		} else if("delete".equals(userDto.getType())) {
			User user = userService.getUserByUserId(Long.parseLong(userDto.getIdValue()));
			userService.doDeleteUser(user);
			logger.info("根据id[" + userDto.getIdValue() + "]删除用户:" + user);
			message += user;
		} else if("in".equals(userDto.getType())) {
			String[] idArray = userDto.getIdValue().split(",");
			List<Long> userIdList = new ArrayList<Long>();
			for(String idString : idArray) {
				userIdList.add(Long.parseLong(idString));
			}
			List<User> userList = userService.queryUsersInUserId(userIdList);
			logger.info("根据id集合[" + userDto.getIdValue() + "]查询用户:" + userList.size());
			for(int i=0;i<userList.size();i++) {
				if(i > 0) {
					message += ",";
				}
				message += userList.get(i);
			}
		} else if("between".equals(userDto.getType())) {
			String[] idArray = userDto.getIdValue().split(",");
			Long beginUserId = Long.parseLong(idArray[0]);
			Long endUserId = Long.parseLong(idArray[1]);
			List<User> userList = userService.queryUsersBetweenUserId(beginUserId, endUserId);
			logger.info("根据id区间[" + userDto.getIdValue() + "]查询用户:" + userList.size());
			for(int i=0;i<userList.size();i++) {
				if(i > 0) {
					message += ",";
				}
				message += userList.get(i);
			}
		} else if("all".equals(userDto.getType())) {
			List<User> userList = userService.queryUsersInUserId(null);
			logger.info("查询所有用户:" + userList.size());
			for(int i=0;i<userList.size();i++) {
				if(i > 0) {
					message += ",";
				}
				message += userList.get(i);
			}
		} else if("count".equals(userDto.getType())) {
			long result = userService.countUser();
			logger.info("查询count:" + result);
			message += "count=[" + result + "]";
		} else if("sum".equals(userDto.getType())) {
			long result = userService.sumUser();
			logger.info("查询sum:" + result);
			message += "sum=[" + result + "]";
		} else if("min".equals(userDto.getType())) {
			long result = userService.minUser();
			logger.info("查询min:" + result);
			message += "min=[" + result + "]";
		} else if("max".equals(userDto.getType())) {
			long result = userService.maxUser();
			logger.info("查询max:" + result);
			message += "max=[" + result + "]";
		} else if("avg".equals(userDto.getType())) {
			double result = userService.avgUser();
			logger.info("查询avg:" + result);
			message += "avg=[" + result + "]";
		} else if("limit".equals(userDto.getType())) {
			String[] idArray = userDto.getIdValue().split(",");
			Long beginUserId = Long.parseLong(idArray[0]);
			Long count = Long.parseLong(idArray[1]);
			List<User> userList = userService.queryUsersLimitUserId(beginUserId, count);
			logger.info("根据limit区间[" + userDto.getIdValue() + "]查询用户:" + userList.size());
			for(int i=0;i<userList.size();i++) {
				if(i > 0) {
					message += ",";
				}
				message += userList.get(i);
			}
		} else {
			/**
			 * 2.查询类型不存在
			 */
			userDto.setSuccess(Boolean.FALSE.booleanValue());
			userDto.setMessage("查询类型不存在!");
			return "user/result";
		}
		
		/**
		 * 3.返回结果
		 */
		userDto.setSuccess(Boolean.TRUE.booleanValue());
		userDto.setMessage(message);
		return "user/result";
	}
}